@@ -44,6 +44,7 @@
 #include "gvfsdaemonprotocol.h"
 #include "gvfskeyring.h"
 #include "gmounttracker.h"
+#include "gvfsbackendsmbprivate.h"
 
 #include <libsmbclient.h>
 
@@ -238,7 +239,7 @@ g_vfs_backend_smb_browse_init (GVfsBackendSmbBrowse *backend)
   g_mutex_init (&backend->update_cache_lock);
 
   if (mount_tracker == NULL)
-    mount_tracker = g_mount_tracker_new (NULL);
+    mount_tracker = g_mount_tracker_new (NULL, FALSE);
 
   /* Get default workgroup name */
   settings = g_settings_new ("org.gnome.system.smb");
@@ -540,56 +541,10 @@ purge_cached (SMBCCTX * context)
   return 0;
 }
 
-#define SUB_DELIM_CHARS  "!$&'()*+,;="
-
-static gboolean
-is_valid (char c, const char *reserved_chars_allowed)
-{
-  if (g_ascii_isalnum (c) ||
-      c == '-' ||
-      c == '.' ||
-      c == '_' ||
-      c == '~')
-    return TRUE;
-
-  if (reserved_chars_allowed &&
-      strchr (reserved_chars_allowed, c) != NULL)
-    return TRUE;
-  
-  return FALSE;
-}
-
-static void
-g_string_append_encoded (GString *string,
-			 const char *encoded,
-			 const char *encoded_end,
-			 const char *reserved_chars_allowed)
-{
-  char c;
-  static const gchar hex[16] = "0123456789ABCDEF";
-
-  if (encoded_end == NULL)
-    encoded_end = encoded + strlen (encoded);
-  
-  while (encoded < encoded_end)
-    {
-      c = *encoded++;
-      
-      if (is_valid (c, reserved_chars_allowed))
-	g_string_append_c (string, c);
-      else
-	{
-	  g_string_append_c (string, '%');
-	  g_string_append_c (string, hex[((guchar)c) >> 4]);
-	  g_string_append_c (string, hex[((guchar)c) & 0xf]);
-	}
-    }
-}
-
 static gboolean
 update_cache (GVfsBackendSmbBrowse *backend, SMBCFILE *supplied_dir)
 {
-  GString *uri;
+  char *uri;
   char dirents[1024*4];
   struct smbc_dirent *dirp;
   GList *entries;
@@ -610,22 +565,14 @@ update_cache (GVfsBackendSmbBrowse *backend, SMBCFILE *supplied_dir)
   g_debug ("update_cache - updating...\n");
   
   /* Update Cache */
-  uri = g_string_new ("smb://");
-
-  if (backend->server)
-    {
-      g_string_append_encoded (uri, backend->server, NULL, NULL);
-      if (backend->port != -1)
-        g_string_append_printf (uri, ":%d", backend->port);
-      g_string_append_c (uri, '/');
-    }
 
   smbc_opendir = smbc_getFunctionOpendir (backend->smb_context);
   smbc_getdents = smbc_getFunctionGetdents (backend->smb_context);
   smbc_closedir = smbc_getFunctionClosedir (backend->smb_context);
 
-  dir = supplied_dir ? supplied_dir : smbc_opendir (backend->smb_context, uri->str);
-  g_string_free (uri, TRUE);
+  uri = create_smb_uri (backend->server, backend->port, NULL, NULL);
+  dir = supplied_dir ? supplied_dir : smbc_opendir (backend->smb_context, uri);
+  g_free (uri);
   if (dir == NULL)
     {
       entry_errno = errno;
@@ -840,7 +787,7 @@ do_mount (GVfsBackend *backend,
   char *icon;
   char *symbolic_icon;
   gchar *port_str;
-  GString *uri;
+  char *uri;
   gboolean res;
   GMountSpec *browse_mount_spec;
   smbc_opendir_fn smbc_opendir;
@@ -952,17 +899,9 @@ do_mount (GVfsBackend *backend,
   smbc_opendir = smbc_getFunctionOpendir (smb_context);
   smbc_closedir = smbc_getFunctionClosedir (smb_context);
 
-  uri = g_string_new ("smb://");
+  uri = create_smb_uri (op_backend->server, op_backend->port, NULL, NULL);
 
-  if (op_backend->server)
-    {
-      g_string_append_encoded (uri, op_backend->server, NULL, NULL);
-      if (op_backend->port != -1)
-        g_string_append_printf (uri, ":%d", op_backend->port);
-      g_string_append_c (uri, '/');
-    }
-
-  g_debug ("do_mount - URI = %s\n", uri->str);
+  g_debug ("do_mount - URI = %s\n", uri);
 
   do
     {
@@ -971,10 +910,10 @@ do_mount (GVfsBackend *backend,
 
       g_debug ("do_mount - try #%d \n", op_backend->mount_try);
 
-      dir = smbc_opendir (smb_context, uri->str);
+      dir = smbc_opendir (smb_context, uri);
 
       g_debug ("do_mount - [%s; %d] dir = %p, cancelled = %d, errno = [%d] '%s' \n",
-             uri->str, op_backend->mount_try, dir, op_backend->mount_cancelled, 
+             uri, op_backend->mount_try, dir, op_backend->mount_cancelled,
              errno, g_strerror (errno));
 
       if (dir == NULL && 
@@ -1012,7 +951,7 @@ do_mount (GVfsBackend *backend,
     }
   while (op_backend->mount_try_again);
 
-  g_string_free (uri, TRUE);
+  g_free (uri);
 
   op_backend->mount_source = NULL;
 
@@ -1312,7 +1251,7 @@ get_file_info_from_entry (GVfsBackendSmbBrowse *backend, BrowseEntry *entry, GFi
 	  entry->smbc_type == SMBC_SERVER)
 	{
 	  uri = g_string_new ("smb://");
-	  g_string_append_encoded (uri, entry->name, NULL, NULL);
+          g_string_append_uri_escaped (uri, entry->name, NULL, FALSE);
 	  g_string_append_c (uri, '/');
 	}
       else
@@ -1320,16 +1259,16 @@ get_file_info_from_entry (GVfsBackendSmbBrowse *backend, BrowseEntry *entry, GFi
 	  mount_spec = get_mount_spec_for_share (backend->server, backend->port, entry->name);
 
 	  uri = g_string_new ("smb://");
-	  g_string_append_encoded (uri, backend->server, NULL, NULL);
+          g_string_append_uri_escaped (uri, backend->server, NULL, FALSE);
 	  g_string_append_c (uri, '/');
-	  g_string_append_encoded (uri, entry->name, NULL, NULL);
+          g_string_append_uri_escaped (uri, entry->name, NULL, FALSE);
 	}
     }
   else
     {
       /* browsing network */
       uri = g_string_new ("smb://");
-      g_string_append_encoded (uri, entry->name, NULL, NULL);
+      g_string_append_uri_escaped (uri, entry->name, NULL, FALSE);
       g_string_append_c (uri, '/');
 
       /* these are auto-mounted, so no CAN_MOUNT/UNMOUNT */
